﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;

namespace PhotoWebSL
{
    public partial class Page : UserControl
    {
        Uri ImageService { get; set; }
        Uri ImageListService { get; set; }

        public Page()
        {
            InitializeComponent();
            ImageService = new Uri(App.Current.Host.Source, "/img");
            ImageListService = new Uri(App.Current.Host.Source, "/images");
            ImageItem.BaseUri = ImageService.ToString();
            Loaded += (s, e) =>
            {
                LoadImages();
            };
        }


        ObservableCollection<ImageItem> ImageDocument { get; set; }

        void LoadImages()
        {
            try
            {
                WebRequest request = WebRequest.Create(ImageListService);
                request.BeginGetResponse(ImagesResponse, request);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }

        void ImagesResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                Stream responseStream = response.GetResponseStream();

                // Get a serializer
                DataContractJsonSerializer ser = new DataContractJsonSerializer(typeof(ImageItem[]));

                // Read the data into an array
                ImageItem[] imageItems = ser.ReadObject(responseStream) as ImageItem[];

                // Update the UI
                ImageDocument = new ObservableCollection<ImageItem>();
                foreach (ImageItem item in imageItems)
                {
                    ImageDocument.Add(item);
                }
                Dispatcher.BeginInvoke(delegate
                {
                    lstImages.DataContext = ImageDocument;
                });
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
            catch (Exception e)
            {
                Dispatcher.BeginInvoke(delegate
                {
                    MessageBox.Show(e.ToString());
                });
            }
        }

        private void DeleteImages_Click(object sender, RoutedEventArgs e)
        {
            if (grdDetails.DataContext != null)
            {
                ImageItem item = (ImageItem)grdDetails.DataContext;
                HtmlPage.Window.Invoke("DeleteImage", item.ImageId);

                ImageDocument.RemoveAt(lstImages.SelectedIndex);
                grdDetails.DataContext = null;
            }
        }

        private void AddImages_Click(object sender, RoutedEventArgs e)
        {
            // Let the user pick a JPEG or PNG to send.
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Multiselect = false;
            ofd.Filter = "JPEG Files|*.jpeg;*.jpg|PNG Files|*.png";
            bool? result = ofd.ShowDialog();
            if (result.HasValue && result.Value)
            {
                // Read the file.
                Stream stream = ofd.File.OpenRead();
                byte[] bytes = new byte[ofd.File.Length];
                stream.Read(bytes, 0, bytes.Length);
                stream.Close();

                // Send the file to the application.
                WebRequest request = WebRequest.Create(ImageListService);
                request.Method = "POST";
                request.ContentType = "image";
                request.BeginGetRequestStream(GetAddImageRequestStreamCallback,
                    new object[] { request, bytes });
            }
        }

        void GetAddImageRequestStreamCallback(IAsyncResult result)
        {
            object[] stateVariables = (object[])result.AsyncState;
            WebRequest request = (WebRequest)stateVariables[0];
            byte[] image = (byte[])stateVariables[1];
            try
            {
                Stream stream = request.EndGetRequestStream(result);
                stream.Write(image, 0, image.Length);
                stream.Close();
                request.BeginGetResponse(AddResponse, request);
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }

        void AddResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                LoadImages();
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }

        private void ListBox_SelectionChanged(object sender,
            SelectionChangedEventArgs e)
        {
            if (grdDetails.DataContext != null && chkPublic.IsEnabled)
            {
                WebRequest request = WebRequest.Create(ImageService);
                request.Method = "POST";
                request.ContentType = "application/json";
                ImageItem imageItem = (ImageItem)grdDetails.DataContext;
                request.BeginGetRequestStream(
                    GetUpdateImageRequestStreamCallback,
                    new object[] { request, imageItem });
            }
            grdDetails.DataContext = ((ListBox)sender).SelectedItem;
        }

        void GetUpdateImageRequestStreamCallback(IAsyncResult result)
        {
            try
            {
                object[] stateInfo = (object[])(result.AsyncState);
                WebRequest request = (WebRequest)(stateInfo[0]);
                ImageItem imageItem = (ImageItem)(stateInfo[1]);
                Stream stream = request.EndGetRequestStream(result);
                DataContractJsonSerializer ser = new
                    DataContractJsonSerializer(typeof(ImageItem));
                ser.WriteObject(stream, imageItem);
                stream.Close();
                request.BeginGetResponse(UpdateResponse, request);
            }
            catch (Exception ex)
            {
                Dispatcher.BeginInvoke(delegate
                {
                    MessageBox.Show(ex.ToString());
                });
            }
        }

        void UpdateResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                response.Close();
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }
    }
}
